#!/usr/bin/python

import sys
import string


######
# This function returns the complementary sequence
######
def comp(raw_sequence):
	uppercase = {'a':'A', 'A':'A', 'c':'C', 'C':'C', 'g':'G', 'G':'G', 't':'T', 'T':'T'}
	complement = {'a':'T', 'A':'T', 'c':'G', 'C':'G', 'g':'C', 'G':'C', 't':'A', 'T':'A'}
	antisense_sequence_array = []
	for letter in raw_sequence:
		if letter in uppercase:
			antisense_sequence_array.insert(0, complement[letter])
	antisense_sequence = ''.join(antisense_sequence_array)
	return antisense_sequence






######
# This function returns the sequence with all unnecessary characters stripped out
######
def stripped_seq(raw_sequence):
	uppercase = {'a':'A', 'A':'A', 'c':'C', 'C':'C', 'g':'G', 'G':'G', 't':'T', 'T':'T'}
	sense_sequence_array = []
	for letter in raw_sequence:
		if letter in uppercase:
			sense_sequence_array.append(uppercase[letter])
	sense_sequence = ''.join(sense_sequence_array)
	return sense_sequence

sys.stdout.write('Pointers to sequences module installed.\n')






######
# This function parses the scaffold into a set of strands
######
def strand_array(scaffold_filename, TPP_ra):
	# Read in scaffold sequence from file
	input_file = file(scaffold_filename, 'r')
	line = input_file.read()
	input_file.close()
	scaf_seq = stripped_seq(line)
	
	# Parse into strands
	strand_ra = []
	start_marker = 0
	num_strands = len(TPP_ra)
	strand_length = len(TPP_ra[0]*7)
	
	for strand_num in range(num_strands):
		end_marker = start_marker + strand_length
		strand_ra.append(scaf_seq[start_marker:end_marker])
		start_marker = end_marker + 12
					
	return strand_ra






######
# This function inputs the strand array and parses it into the token array
######
def token_array(strand_ra):
	token_ra = []
	num_strands = len(strand_ra)
	num_subzones = len(strand_ra[0])/7
	
	for strand_num in range(num_strands):
		sub_token_ra = []
		for subzone_num in range(num_subzones):
			token = comp(strand_ra[strand_num][subzone_num*7:subzone_num*7 + 7])
			if strand_num%2 == 0:
				sub_token_ra.append(token)
			else:
				sub_token_ra.insert(0, token)
		token_ra.append(sub_token_ra)
	
	return token_ra






######
# This function inputs the OTP array and the token array, and assigns sequences to the oligos
######
def oligo_array(OTP_ra, token_ra):
	oligo_ra = []
	for sub_OTP_ra in OTP_ra:
		oligo = ''
		for TP in sub_OTP_ra:
			strand = TP[0]
			subzone = TP[1]
			oligo += token_ra[strand][subzone]
		oligo_ra.append(oligo)
	
	return oligo_ra






sys.stdout.write('Pointers to sequences module installed.\n')
